"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MidwayKafkaFramework = void 0;
const core_1 = require("@midwayjs/core");
const decorator_1 = require("@midwayjs/decorator");
const kafka_1 = require("./kafka");
let MidwayKafkaFramework = class MidwayKafkaFramework extends core_1.BaseFramework {
    configure() {
        return this.configService.getConfiguration('kafka');
    }
    async applicationInitialize(options) {
        // Create a connection manager
        this.app = new kafka_1.KafkaConsumerServer({
            logger: this.logger,
            ...this.configurationOptions,
        });
    }
    async run() {
        await this.app.connect(this.configurationOptions.kafkaConfig, this.configurationOptions.consumerConfig);
        await this.loadSubscriber();
        this.logger.info('Kafka consumer server start success');
    }
    async loadSubscriber() {
        const subscriberModules = (0, decorator_1.listModule)(decorator_1.MS_CONSUMER_KEY, module => {
            const metadata = (0, decorator_1.getClassMetadata)(decorator_1.MS_CONSUMER_KEY, module);
            return metadata.type === decorator_1.MSListenerType.KAFKA;
        });
        for (const module of subscriberModules) {
            const data = (0, decorator_1.listPropertyDataFromClass)(decorator_1.MS_CONSUMER_KEY, module);
            for (const methodBindListeners of data) {
                for (const listenerOptions of methodBindListeners) {
                    await this.app.connection.subscribe({
                        topic: listenerOptions.topic,
                        // fromBeginning: true,
                    });
                    await this.app.connection.run({
                        autoCommit: false,
                        eachMessage: async ({ topic, partition, message }) => {
                            const ctx = {
                                topic: topic,
                                partition: partition,
                                message: message,
                                commitOffsets: (offset) => {
                                    return this.app.connection.commitOffsets([
                                        {
                                            topic: topic,
                                            partition: partition,
                                            offset,
                                        },
                                    ]);
                                },
                            };
                            this.app.createAnonymousContext(ctx);
                            const ins = await ctx.requestContext.getAsync(module);
                            const fn = await this.applyMiddleware(async (ctx) => {
                                return await ins[listenerOptions.propertyKey].call(ins, message);
                            });
                            try {
                                const result = await fn(ctx);
                                // 返回为undefined，下面永远不会执行
                                if (result) {
                                    const res = await this.app.connection.commitOffsets(message.offset);
                                    return res;
                                }
                            }
                            catch (error) {
                                this.logger.error(error);
                            }
                        },
                    });
                }
            }
        }
    }
    getFrameworkName() {
        return 'midway:kafka';
    }
    async beforeStop() {
        await this.app.close();
    }
    getFrameworkType() {
        return decorator_1.MidwayFrameworkType.MS_KAFKA;
    }
};
MidwayKafkaFramework = __decorate([
    (0, decorator_1.Framework)()
], MidwayKafkaFramework);
exports.MidwayKafkaFramework = MidwayKafkaFramework;
//# sourceMappingURL=framework.js.map